# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import OrderedDict
import functools
import re
from typing import Dict, Mapping, MutableMapping, MutableSequence, Optional, Sequence, Tuple, Type, Union

from oceanbolt.com.portcalls_v3 import gapic_version as package_version

from google.api_core.client_options import ClientOptions
from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1
from google.api_core import retry as retries
from google.auth import credentials as ga_credentials   # type: ignore
from google.oauth2 import service_account              # type: ignore

try:
    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.Retry, object]  # type: ignore

from oceanbolt.com.portcalls_v3.types import service
from .transports.base import PortCallServiceTransport, DEFAULT_CLIENT_INFO
from .transports.grpc_asyncio import PortCallServiceGrpcAsyncIOTransport
from .client import PortCallServiceClient


class PortCallServiceAsyncClient:
    """PortCallService provides an API service to get PortCall data"""

    _client: PortCallServiceClient

    DEFAULT_ENDPOINT = PortCallServiceClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = PortCallServiceClient.DEFAULT_MTLS_ENDPOINT

    common_billing_account_path = staticmethod(PortCallServiceClient.common_billing_account_path)
    parse_common_billing_account_path = staticmethod(PortCallServiceClient.parse_common_billing_account_path)
    common_folder_path = staticmethod(PortCallServiceClient.common_folder_path)
    parse_common_folder_path = staticmethod(PortCallServiceClient.parse_common_folder_path)
    common_organization_path = staticmethod(PortCallServiceClient.common_organization_path)
    parse_common_organization_path = staticmethod(PortCallServiceClient.parse_common_organization_path)
    common_project_path = staticmethod(PortCallServiceClient.common_project_path)
    parse_common_project_path = staticmethod(PortCallServiceClient.parse_common_project_path)
    common_location_path = staticmethod(PortCallServiceClient.common_location_path)
    parse_common_location_path = staticmethod(PortCallServiceClient.parse_common_location_path)

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            PortCallServiceAsyncClient: The constructed client.
        """
        return PortCallServiceClient.from_service_account_info.__func__(PortCallServiceAsyncClient, info, *args, **kwargs)  # type: ignore

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            PortCallServiceAsyncClient: The constructed client.
        """
        return PortCallServiceClient.from_service_account_file.__func__(PortCallServiceAsyncClient, filename, *args, **kwargs)  # type: ignore

    from_service_account_json = from_service_account_file

    @classmethod
    def get_mtls_endpoint_and_cert_source(cls, client_options: Optional[ClientOptions] = None):
        """Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variable is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """
        return PortCallServiceClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore

    @property
    def transport(self) -> PortCallServiceTransport:
        """Returns the transport used by the client instance.

        Returns:
            PortCallServiceTransport: The transport used by the client instance.
        """
        return self._client.transport

    get_transport_class = functools.partial(type(PortCallServiceClient).get_transport_class, type(PortCallServiceClient))

    def __init__(self, *,
            credentials: Optional[ga_credentials.Credentials] = None,
            transport: Union[str, PortCallServiceTransport] = "grpc_asyncio",
            client_options: Optional[ClientOptions] = None,
            client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
            ) -> None:
        """Instantiates the port call service client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Union[str, ~.PortCallServiceTransport]): The
                transport to use. If set to None, a transport is chosen
                automatically.
            client_options (ClientOptions): Custom options for the client. It
                won't take effect if a ``transport`` instance is provided.
                (1) The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client. GOOGLE_API_USE_MTLS_ENDPOINT
                environment variable can also be used to override the endpoint:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto switch to the
                default mTLS endpoint if client certificate is present, this is
                the default value). However, the ``api_endpoint`` property takes
                precedence if provided.
                (2) If GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide client certificate for mutual TLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = PortCallServiceClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,

        )

    async def get_port_calls(self,
            request: Optional[Union[service.GetPortCallsRequest, dict]] = None,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Union[float, object] = gapic_v1.method.DEFAULT,
            metadata: Sequence[Tuple[str, str]] = (),
            ) -> service.GetPortCallsResponse:
        r"""ListPortCalls retrieves lists all port calls for a
        set of request params. Response is paginated, and
        endpoint accepts a paging parameter to specify which
        page to return. It is also possible to set the number of
        port calls to return per query, default is 50 and
        maximum is 2000.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from oceanbolt.com import portcalls_v3

            async def sample_get_port_calls():
                # Create a client
                client = portcalls_v3.PortCallServiceAsyncClient()

                # Initialize request argument(s)
                request = portcalls_v3.GetPortCallsRequest(
                )

                # Make the request
                response = await client.get_port_calls(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[oceanbolt.com.portcalls_v3.types.GetPortCallsRequest, dict]]):
                The request object. Port calls data requests object.
                This is shared between all port calls queries
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            oceanbolt.com.portcalls_v3.types.GetPortCallsResponse:
                Response object for port call queries
        """
        # Create or coerce a protobuf request object.
        request = service.GetPortCallsRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.get_port_calls,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def get_port_call_timeseries(self,
            request: Optional[Union[service.GetPortCallsRequest, dict]] = None,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Union[float, object] = gapic_v1.method.DEFAULT,
            metadata: Sequence[Tuple[str, str]] = (),
            ) -> service.GetPortCallTimeseriesResponse:
        r"""GetPortCallTimeseries retrieves aggregated counts for
        port calls for a set of request params.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from oceanbolt.com import portcalls_v3

            async def sample_get_port_call_timeseries():
                # Create a client
                client = portcalls_v3.PortCallServiceAsyncClient()

                # Initialize request argument(s)
                request = portcalls_v3.GetPortCallsRequest(
                )

                # Make the request
                response = await client.get_port_call_timeseries(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[oceanbolt.com.portcalls_v3.types.GetPortCallsRequest, dict]]):
                The request object. Port calls data requests object.
                This is shared between all port calls queries
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            oceanbolt.com.portcalls_v3.types.GetPortCallTimeseriesResponse:
                Response object for port call
                timeseries queries

        """
        # Create or coerce a protobuf request object.
        request = service.GetPortCallsRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.get_port_call_timeseries,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def get_port_particulars(self,
            request: Optional[Union[service.GetPortParticularsRequest, dict]] = None,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Union[float, object] = gapic_v1.method.DEFAULT,
            metadata: Sequence[Tuple[str, str]] = (),
            ) -> service.GetPortParticularsResponse:
        r"""ListPortCalls retrieves lists all port calls for a
        given vessel. Response is paginated, and endpoint
        accepts a paging parameter to specify which page to
        return. It is also possible to set the number of port
        calls to return per query, default is 50 and maximum is
        2000.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from oceanbolt.com import portcalls_v3

            async def sample_get_port_particulars():
                # Create a client
                client = portcalls_v3.PortCallServiceAsyncClient()

                # Initialize request argument(s)
                request = portcalls_v3.GetPortParticularsRequest(
                )

                # Make the request
                response = await client.get_port_particulars(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[oceanbolt.com.portcalls_v3.types.GetPortParticularsRequest, dict]]):
                The request object. Request object for
                GetPortParticulars
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            oceanbolt.com.portcalls_v3.types.GetPortParticularsResponse:
                Response object for
                GetPortParticulars

        """
        # Create or coerce a protobuf request object.
        request = service.GetPortParticularsRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.get_port_particulars,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def get_vessels_in_port(self,
            request: Optional[Union[service.GetVesselsInPortRequest, dict]] = None,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Union[float, object] = gapic_v1.method.DEFAULT,
            metadata: Sequence[Tuple[str, str]] = (),
            ) -> service.GetVesselsInPortResponse:
        r"""GetVesselsInPort lists all vessels that were within a
        port (or related anchorages) at a certain time.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from oceanbolt.com import portcalls_v3

            async def sample_get_vessels_in_port():
                # Create a client
                client = portcalls_v3.PortCallServiceAsyncClient()

                # Initialize request argument(s)
                request = portcalls_v3.GetVesselsInPortRequest(
                )

                # Make the request
                response = await client.get_vessels_in_port(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[oceanbolt.com.portcalls_v3.types.GetVesselsInPortRequest, dict]]):
                The request object. Request object for GetVesselsInPort
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            oceanbolt.com.portcalls_v3.types.GetVesselsInPortResponse:
                Response object for GetVesselsInPort
        """
        # Create or coerce a protobuf request object.
        request = service.GetVesselsInPortRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.get_vessels_in_port,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.transport.close()

DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(gapic_version=package_version.__version__)


__all__ = (
    "PortCallServiceAsyncClient",
)
